###--- >>> `xmlTreeParse' <<<----- XML Parser

	## alias	 help(xmlTreeParse)

##___ Examples ___:

 fileName <- system.file("data", "test.xml", pkg="XML")
   # parse the document and return it in its standard format.
 xmlTreeParse(fileName)

   # parse the document, discarding comments.
  
 xmlTreeParse(fileName, handlers=list("comment"=function(x, parent){NULL}))

 invisible(xmlTreeParse(fileName,
            handlers=list(entity=function(x) {
                                    cat("In entity",x$name, x$value,"\n")}
                                  )
                          )
         )

 # Parse some XML text.
 # Read the text from the file
 xmlText <- paste(scan(fileName, what="",sep="\n"),"\n", collapse="\n")
 xmlTreeParse(xmlText, asText=T)

 # Read a MathML document and convert each node
 # so that the primary class is 
 #   <name of tag>MathML
 # so that we can use method  dispatching when processing
 # it rather than conditional statements on the tag name.
 # See plotMathML() in examples/.
 fileName <- system.file("data", "mathml.xml",pkg="XML")
m <- xmlTreeParse(fileName, 
                  handlers=list(startElement=function(node){
                              cname <- paste(xmlName(node),"MathML",sep="",collapse="")
                              class(node) <- c(cname, class(node)); 
                              node
                }))


  # In this example, we extract _just_ the names of the
  # variables in the mtcars.xml file. 
  # The names are the contents of the <variable>
  # tags. We discard all other tags by returning NULL
  # from the startElement handler.
  #
  # We cumulate the names of variables in a character
  # vector named `vars'.
  # We define this within a closure and define the 
  # variable function within that closure so that it
  # will be invoked when the parser encounters a <variable>
  # tag.
  # This is called with 2 arguments: the XMLNode object (containing
  # its children) and the list of attributes.
  # We get the variable name via call to xmlValue().

  # Note that we define the closure function in the call and then 
  # create an instance of it by calling it directly as
  #   (function() {...})()

  # Note that we can get the names by parsing
  # in the usual manner and the entire document and then executing
  # xmlSApply(xmlRoot(doc)[[1]], function(x) xmlValue(x[[1]]))
  # which is simpler but is more costly in terms of memory.
 fileName <- system.file("data", "mtcars.xml")
 doc <- xmlTreeParse(fileName,  handlers = (function() { 
                                 vars <- character(0) ;
                                list(variable=function(x, attrs) { 
                                                vars <<- c(vars, xmlValue(x[[1]])); 
                                                NULL}, 
                                     startElement=function(x,attr){
                                                   NULL
                                                  }, 
                                     names = function() {
                                                 vars
                                             }
                                    )
                               })()
                     )

  # Here we just print the variable names to the console
  # with a special handler.
 doc <- xmlTreeParse(fileName, handlers = list(
                                  variable=function(x, attrs) {
                                             print(xmlValue(x[[1]])); T
                                           }))


##Don't run: 
##D  # Parse an XML document directly from a URL.
##D  # Requires Internet access.
##D  xmlTreeParse("http://www.omegahat.org/Scripts/Data/mtcars.xml", asText=T)


## Keywords: 'file', 'XML', 'DTD'.


